import React, { useMemo } from 'react';
import {
 Route, Redirect, Switch, useRouteMatch,
} from 'react-router-dom';
import LazyComponent from '@Components/Base/Lazy';
import BaseRoutes from '@Base/routes';
import { RouteItem } from '@Base/routes/types';
import useMappedState from './useMappedState';

type useMappedRouteProps = React.ReactNode;

/**
 * 描述 : 取得某個 Route 底下的 SubRoute
 */

export default function useMappedRoute(
  routeLayer: string[],
  signInRedirect = '',
  noSignInRedirect = '',
): useMappedRouteProps {
  /* Global & Local State */
  const routeMatch = useRouteMatch();
  const storeUser = useMappedState((state) => state.user);
  /* Data */
  const ReturnRoutes: RouteItem[] = useMemo(() => {
    let fullRoutes: RouteItem[] = [];
    let result: RouteItem[] | null = null;
    let hadResult = false;
    if (storeUser.userIsLogin) {
      Object.values(BaseRoutes.Admin).forEach((route) => {
        fullRoutes = fullRoutes.concat(route);
      });
    } else {
      Object.values(BaseRoutes.Guest).forEach((route) => {
        fullRoutes = fullRoutes.concat(route);
      });
    }
    if (routeLayer.length === 0) {
      result = fullRoutes;
    }
    for (let i = 0; i < routeLayer.length; i += 1) {
      if (!hadResult) {
        const recentRouteObject = fullRoutes.find((route) => route.name === routeLayer[i]);
        if (recentRouteObject) {
          result = recentRouteObject.subRoute;
          hadResult = true;
        }
      }
      if (hadResult && result) {
        const recentRouteObject = result.find((route) => route.name === routeLayer[i]);
        if (recentRouteObject) {
          result = recentRouteObject.subRoute;
        }
      }
    }
    if (result) {
      return result;
    }
    return [];
  }, [storeUser.userIsLogin]);
  const ReturnReDirect = useMemo<string>(() => {
    if (storeUser.userIsLogin) {
      if (ReturnRoutes.length > 0) {
        return ReturnRoutes[0].route;
      }
    }
    if (ReturnRoutes.length > 0) {
      return ReturnRoutes[0].route;
    }
    return '/';
  }, [storeUser.userIsLogin, signInRedirect, noSignInRedirect]);
  /* View */
  const RenderRoutes = useMemo(
    () => (
      <Switch>
        <Route
          exact
          path={`${routeMatch.url.replace('/', '') === '' ? '/' : `/${routeMatch.url.replace('/', '')}`}`}
          render={() => (
            <Redirect
              to={`${
                routeMatch.url.replace('/', '') === '' ? '' : `/${routeMatch.url.replace('/', '')}`
              }${ReturnReDirect}`}
            />
          )}
        />
        {ReturnRoutes.map((route) => (
          <Route
            key={route.route}
            exact={route.exact}
            path={`${routeMatch.path.replace('/', '') === '' ? '' : `/${routeMatch.path.replace('/', '')}`}${
              route.route
            }`}
            render={() => (
              <LazyComponent
                componentImport={route.component}
                componentChunkName={`${route.name}Chunk`}
                componentProps={{}}
              />
            )}
          />
        ))}
        <Redirect
          to={`${routeMatch.url.replace('/', '') === '' ? '' : `/${routeMatch.url.replace('/', '')}`}${ReturnReDirect}`}
        />
      </Switch>
    ),
    [storeUser.userIsLogin, routeMatch, ReturnReDirect],
  );
  /* Main */
  return <>{RenderRoutes}</>;
}
